"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var ts = require("typescript");
var through = require('through2');
/**
 * attach class Annotations before typescript ts compile.
 *
 * @export
 * @param {string} [annotationField='classAnnations']
 * @returns
 */
function classAnnotations(annotationField) {
    if (annotationField === void 0) { annotationField = 'classAnnations'; }
    return through.obj(function (file, encoding, callback) {
        if (file.isNull()) {
            return callback(null, file);
        }
        if (file.isStream()) {
            return callback('doesn\'t support Streams');
        }
        var contents = file.contents.toString('utf8');
        // fix typescript '$' bug when create source file.
        contents = contents.replace(/\'\$\'/gi, '"$"');
        var sourceFile = ts.createSourceFile('cache.source.ts', contents, ts.ScriptTarget.Latest, true);
        var eachChild = function (node, annations) {
            if (ts.isClassDeclaration(node)) {
                var className = node.name.text;
                var annations_1 = {
                    name: className,
                    params: {}
                };
                var oldclass = node.getText();
                ts.forEachChild(node, function (node) { return eachChild(node, annations_1); });
                var classAnnations = "\n                        static " + annotationField + ":any  = " + JSON.stringify(annations_1) + ";\n                   ";
                var end = oldclass.replace(/\s*$/, '').length - 1;
                contents = contents.replace(oldclass, oldclass.substring(0, end) + classAnnations + oldclass.substring(end));
            }
            else if (ts.isConstructorDeclaration(node)) {
                if (annations) {
                    var paramNames = node.parameters.map(function (param) {
                        return param.name.text;
                    });
                    annations.params['constructor'] = paramNames;
                }
            }
            else if (ts.isMethodDeclaration(node)) {
                if (annations) {
                    var paramNames = node.parameters.map(function (param) {
                        return param.name.text;
                    });
                    var method = node.name.text;
                    annations.params[method] = paramNames;
                }
            }
        };
        ts.forEachChild(sourceFile, eachChild);
        file.contents = new Buffer(contents);
        this.push(file);
        callback();
    });
}
exports.classAnnotations = classAnnotations;
// const STRIP_COMMENTS = /((\/\/.*$)|(\/\*[\s\S]*?\*\/))/mg;
// const ARGUMENT_NAMES = /([^\s,]+)/g;
// function getParamNames(func: Function | string) {
//     if (!isFunction(func) || isString(func)) {
//         return [];
//     }
//     let fnStr = func.toString().replace(STRIP_COMMENTS, '');
//     let result = fnStr.slice(fnStr.indexOf('(') + 1, fnStr.indexOf(')')).match(ARGUMENT_NAMES);
//     if (result === null) {
//         result = [];
//     }
//     return result;
// }

//# sourceMappingURL=sourcemaps/classAnnotations.js.map
